﻿<#
.SYNOPSIS
	Automates the download and installation of language packs for Office Web Apps Server 2013

.DESCRIPTION
	Automates the download and installation of language packs for Office Web Apps Server 2013. A GUI pick list showing language packs that are not yet installed. Once picked, language packs are downloaded mounted/extracted and installed.

.NOTES
  Version							: 1.0 - See changelog at http://www.ehloworld.com/2970
	Wish list						: Better error trapping
  Rights Required			: Local administrator on server
  Sched Task Required	: No
  Lync Server Version	: N/A
  Author/Copyright		: © Pat Richard, Lync MVP - All Rights Reserved
  Email/Blog/Twitter	: pat@innervation.com 	http://www.ehloworld.com @patrichard
  Dedicated Post			: http://www.ehloworld.com/2969
  Disclaimer   				: You running this script means you won't blame me if this breaks your stuff. This script is
  											provided AS IS without warranty of any kind. I disclaim all implied warranties including,
  											without limitation, any implied warranties of merchantability or of fitness for a particular
  											purpose. The entire risk arising out of the use or performance of the sample scripts and
  											documentation remains with you. In no event shall I be liable for any damages whatsoever
  											(including, without limitation, damages for loss of business profits, business interruption,
  											loss of business information, or other pecuniary loss) arising out of the use of or inability
  											to use the script or documentation.
  Acknowledgements 		:
  Assumptions					: ExecutionPolicy of AllSigned (recommended), RemoteSigned or Unrestricted (not recommended)
  Limitations					:
  Known issues				: Downloading files through a proxy is still somewhat of a kludge, especially if the proxy isn't configured in IE.

.LINK
	http://www.ehloworld.com/2969

.EXAMPLE
  .\Install-OWASLanguagePacks.ps1

  Description
	-----------
	Runs script with default values.

.INPUTS
	None. You cannot pipe objects to this script.
#>
#Requires -Version 3.0

[CmdletBinding(SupportsShouldProcess = $True, SupportsPaging = $True)]
param(
	[Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $True)]
	$TargetFolder = "c:\_Install",
	$OWASLanguagePackFolder = "$TargetFolder\OWASLanguagePacks"
)

#region Variables
[string] $ScriptPathAndName = $MyInvocation.MyCommand.Definition
[string] $LogPath = "$TargetFolder\logs\$env:ComputerName" + " {0:yyyy-MM-dd hh-mmtt}.log" -f (Get-Date)
[string] $LogDivider = "------------------------------"
#endregion Variables

#region Functions
function New-FileDownload {
	<#
	.SYNOPSIS
		Downloads a file from a specified URL.

	.DESCRIPTION
		Downloads a file from a specified URL. First, it verifies the file does not exist locally. Then ensures Internet access is available, then downloads the file.

	.NOTES
	  Version							: 1.3
		Wish list						: Better error trapping
	  Rights Required			: Local administrator on server
	  Sched Task Required	: No
	  Lync Server Version	: N/A
	  Author/Copyright		: © Pat Richard, Lync MVP - All Rights Reserved
	  Email/Blog/Twitter	: pat@innervation.com 	http://www.ehloworld.com @patrichard
	  Dedicated Post			:
	  Disclaimer   				: You running this script means you won't blame me if this breaks your stuff. This script is
	  											provided AS IS without warranty of any kind. I disclaim all implied warranties including,
	  											without limitation, any implied warranties of merchantability or of fitness for a particular
	  											purpose. The entire risk arising out of the use or performance of the sample scripts and
	  											documentation remains with you. In no event shall I be liable for any damages whatsoever
	  											(including, without limitation, damages for loss of business profits, business interruption,
	  											loss of business information, or other pecuniary loss) arising out of the use of or inability
	  											to use the script or documentation.
	  Acknowledgements 		:
	  Assumptions					: ExecutionPolicy of AllSigned (recommended), RemoteSigned or Unrestricted (not recommended)
	  Limitations					:
	  Known issues				:

	.LINK


	.EXAMPLE
	  .\

	  Description
		-----------


	.INPUTS
		None. You cannot pipe objects to this script.
	#>
	[CmdletBinding(SupportsShouldProcess = $True, SupportsPaging = $True)]
	param(
		# Complete path and file name to the file to be downloaded
		[parameter(ValueFromPipeline = $false, ValueFromPipelineByPropertyName = $true)]
		[ValidateNotNullOrEmpty()]
		[string] $SourceFile,

		# The folder where the downloaded file should be placed. If not defined, it defaults to $TargetFolder.
		[parameter(ValueFromPipeline = $false, ValueFromPipelineByPropertyName = $true)]
		[string] $DestFolder,

		# The file name the downloaded file should be changed to. If not defined, file maintains it's original name.
		[parameter(ValueFromPipeline = $false, ValueFromPipelineByPropertyName = $true)]
		[string] $DestFile,

		# Whether to download even if a local copy exists. This is useful for files that are updated often and need to be redownloaded.
		[parameter(ValueFromPipeline = $false, ValueFromPipelineByPropertyName = $true)]
		[switch] $IgnoreLocalCopy
	)
	[bool] $HasInternetAccess = ([Activator]::CreateInstance([Type]::GetTypeFromCLSID([Guid]'{DCB00C01-570F-4A9B-8D69-199FDBA5723B}')).IsConnectedToInternet)
	Test-IsProxyEnabled
	# I should switch this to using a param block and pipelining from property name - just for consistency

	if (-not($DestFolder)){
		$DestFolder = $TargetFolder
	}
	Set-ModuleStatus -name BitsTransfer
	if (-not($DestFile)){
		[string] $DestFile = ($SourceFile | Split-Path -leaf)
		[Reflection.Assembly]::LoadWithPartialName("System.Web") | Out-Null
		$DestFile = [System.Web.HttpUtility]::UrlDecode($DestFile)
	}
	if (Test-Path $DestFolder){
		Write-Log -Message "Target folder `"$DestFolder`" exists - no need to create" -NoConsole -Indent 1
	} else {
		Write-Log -Message "Folder `"$DestFolder`" does not exist, creating..." -NoConsole
		New-Item $DestFolder -type Directory | Out-Null
		Write-Log -Message "Done!" -NoConsole -Indent 1
	}
	if ((Test-Path "$DestFolder\$DestFile") -and (-not $IgnoreLocalCopy)){
		if ($(((Get-Item $DestFolder\$DestFile).VersionInfo).FileVersion)){
			Write-Log -Message "File `"$DestFile`" version $(((Get-Item $DestFolder\$DestFile).VersionInfo).FileVersion) exists locally - no need to download" -NoConsole
		}else{
			Write-Log -Message "File `"$DestFile`" exists locally - no need to download" -NoConsole
		}
	} else {
		if ($HasInternetAccess){
			Write-Log -Message "Internet access available" -NoConsole -Indent 1
			if (-not $IgnoreLocalCopy){
				Write-Log -Message "File `"$DestFile`" does not exist in `"$DestFolder`"" -NoConsole -Indent 1
			} else {
				Write-Log -Message "Forcing download of `"$DestFile`" to `"$DestFolder`"" -NoConsole
			}
			Write-Log -Message "Downloading `"$SourceFile`" to `"$DestFolder`"" -NoConsole -Indent 1
			########################################################################################################
			# NOTE: Default parameters may have been changed due to proxy settings. See Test-IsProxyEnabled function
			########################################################################################################
			# determine file size before downloading
			<#
			$clnt = New-Object System.Net.WebClient
			$clnt.OpenRead($SourceFile) | Out-Null
			$dlfilesize = [int] $($clnt.ResponseHeaders["Content-Length"]/1mb)
			# $dlfilesize
			$clnt.Dispose()
			#>
			if ($dlfilesize){
				Start-BitsTransfer -Source "$SourceFile" -Destination "$DestFolder\$DestFile" -Description "Downloading $DestFile ($dlfilesize MB)" -ErrorAction SilentlyContinue
			}else{
				Start-BitsTransfer -Source "$SourceFile" -Destination "$DestFolder\$DestFile" -Description "Downloading $DestFile" -ErrorAction SilentlyContinue
			}

			if (Test-Path $DestFolder\$DestFile){
				if ($(((Get-Item $DestFolder\$DestFile).VersionInfo).FileVersion)){
					Write-Log -Message "Successfully downloaded $DestFolder\$DestFile version $(((Get-Item $DestFolder\$DestFile).VersionInfo).FileVersion)" -NoConsole -Indent 1
				}else{
					Write-Log -Message "Successfully downloaded $DestFolder\$DestFile" -NoConsole -Indent 1
				}
			} else {
				Write-Log -Level error -message "Failed! File not downloaded!" -Indent 1
				Write-Log -Message "Prompting user to abort/retry/ignore" -NoConsole
				switch (New-Popup -message "A file download failure has occurred. What would you like to do?" -Title "Download error!" -Buttons "AbortRetryIgnore" -Icon Exclamation){
			    3{ # abort
			    	Write-Log -Message "User has chosen to abort script" -NoConsole -Indent 1
						Write-Log -Message "v$Version script end" -EventLogName application -EventId 1 -NoConsole
						Set-ElapsedTime
						Set-RunningStatus -Stopped
						exit
					}
			    4{ # retry
			    	Write-Log -Message "User has chosen to retry" -NoConsole -Indent 1
			    	Write-Log -Message "Building retry expression" -NoConsole -Indent 2
			    	if ($IgnoreLocalCopy){
			    		$DownloadRetry += " -IgnoreLocalCopy"
			    	}
			    	if ($DestFile){
			    		$DownloadRetry += " -DestFile $Destfile"
			    	}
			    	if ($DestFolder){
			    		$DownloadRetry += " -DestFolder $DestFolder"
			    	}
			    	$DownloadRetryExp = "New-FileDownload -SourceFile $SourceFile"+ $DownloadRetry
			    	Write-Log -Message "Retry expression is $DownloadRetryExp" -NoConsole -Indent 2
			    	Invoke-Expression $DownloadRetryExp
			    }
			    5{ # ignore
			    	Write-Log -Message "User has chosen to ignore" -NoConsole -Indent 1
			    }
				}
			}
		} else {
			Write-Log -Level warn -Message "Internet access not detected."
			Write-Log -Level warn -Message "This can be because there is no Internet connection,"
			Write-Log -Level warn -Message "there is no DNS resolution,"
			Write-Log -Level warn -Message "or a proxy is in place. Please resolve and try again."
			Write-Log -Level warn -Message "Alternatively, you can manually download the file ($SourceFile)"
			Write-Log -Level warn -Message "and place it in $DestFolder and try again."
		}
	}
} # end function New-FileDownload

function Set-ModuleStatus {
	<#
	.SYNOPSIS
		Imports a specified PowerShell module, with error checking.

	.DESCRIPTION
		Imports a specified PowerShell module, with error checking.

	.NOTES
	  Version							: 1.5
		Wish list						: Better error trapping
	  Rights Required			: None
	  Sched Task Required	: No
	  Lync Server Version	: N/A
	  Author/Copyright		: © Pat Richard, Lync MVP - All Rights Reserved
	  Email/Blog/Twitter	: pat@innervation.com 	http://www.ehloworld.com @patrichard
	  Dedicated Post			:
	  Disclaimer   				: You running this script means you won't blame me if this breaks your stuff. This script is
	  											provided AS IS without warranty of any kind. I disclaim all implied warranties including,
	  											without limitation, any implied warranties of merchantability or of fitness for a particular
	  											purpose. The entire risk arising out of the use or performance of the sample scripts and
	  											documentation remains with you. In no event shall I be liable for any damages whatsoever
	  											(including, without limitation, damages for loss of business profits, business interruption,
	  											loss of business information, or other pecuniary loss) arising out of the use of or inability
	  											to use the script or documentation.
	  Acknowledgements 		:
	  Assumptions					: ExecutionPolicy of AllSigned (recommended), RemoteSigned or Unrestricted (not recommended)
	  Limitations					:
	  Known issues				:

	.LINK


	.EXAMPLE
	  .\

	  Description
		-----------


	.INPUTS
		None. You cannot pipe objects to this script.
	#>
	[CmdletBinding(SupportsShouldProcess = $True, SupportsPaging = $True)]
	param	(
		[parameter(ValueFromPipeline = $true, ValueFromPipelineByPropertyName = $true, Mandatory = $true, HelpMessage = "No module name specified!")]
		[ValidateNotNullOrEmpty()]
		[string] $name
	)
	PROCESS{
		# Executes once for each pipeline object
		# the $_ variable represents the current input object
		if (-not(Get-Module -name "$name")) {
			if (Get-Module -ListAvailable | Where-Object Name -eq "$name") {
				Import-Module -Name "$name"
				# module was imported
				# return $true
			} else {
				# module was not available
				# return $false
			}
		} else {
			# Write-Output "$_ module already imported"
			# return $true
		}
	} # end PROCESS
} # end function Set-ModuleStatus

function Test-IsProxyEnabled {
	<#
	.SYNOPSIS
		Determines (the best it can) if a proxy is in place for Internet access.

	.DESCRIPTION
		Determines (the best it can) if a proxy is in place for Internet access. If so, other functions that download files can better handle downloads.

	.NOTES
	  Version							: 1.1
		Wish list						: Better error trapping
	  Rights Required			: Local administrator on server
	  Sched Task Required	: No
	  Lync Server Version	: N/A
	  Author/Copyright		: © Pat Richard, Lync MVP - All Rights Reserved
	  Email/Blog/Twitter	: pat@innervation.com 	http://www.ehloworld.com @patrichard
	  Dedicated Post			:
	  Disclaimer   				: You running this script means you won't blame me if this breaks your stuff. This script is
	  											provided AS IS without warranty of any kind. I disclaim all implied warranties including,
	  											without limitation, any implied warranties of merchantability or of fitness for a particular
	  											purpose. The entire risk arising out of the use or performance of the sample scripts and
	  											documentation remains with you. In no event shall I be liable for any damages whatsoever
	  											(including, without limitation, damages for loss of business profits, business interruption,
	  											loss of business information, or other pecuniary loss) arising out of the use of or inability
	  											to use the script or documentation.
	  Acknowledgements 		:
	  Assumptions					: ExecutionPolicy of AllSigned (recommended), RemoteSigned or Unrestricted (not recommended)
	  Limitations					:
	  Known issues				:

	.LINK


	.EXAMPLE
	  .\

	  Description
		-----------


	.INPUTS
		None. You cannot pipe objects to this script.
	#>
	[CmdletBinding(SupportsShouldProcess = $True, SupportsPaging = $True)]
	param ()
	Write-Log -Message "Checking if a proxy is enabled" -NoConsole -Indent 1
	# If this registry key doesn't exist at all, it's because IE has never been launched. We need to create it here so that download attempts and proxy checks don't throw a silent error
	if ((Get-Item "HKCU:\Software\Microsoft\Windows\CurrentVersion\Internet Settings").Property -notcontains "ProxyEnable"){
		Write-Log -Message "Internet Explorer has never been opened. Proxy registry values have yet to be set. Setting it to 0 now." -NoConsole -Indent 1
		New-ItemProperty "HKCU:\Software\Microsoft\Windows\CurrentVersion\Internet Settings" -Name ProxyEnable -Value 0 | Out-Null
	}
	if ((Get-ItemProperty "HKCU:\Software\Microsoft\Windows\CurrentVersion\Internet Settings" -name ProxyEnable -ErrorAction SilentlyContinue).ProxyEnable -ne 0){
		Write-Log -Message "A proxy is required" -NoConsole -Indent 2
		[string]$ProxyServer = (Get-ItemProperty "HKCU:\Software\Microsoft\Windows\CurrentVersion\Internet Settings" -name ProxyServer).ProxyServer
		Write-Log -Message "Configured proxy: `"$ProxyServer`"" -NoConsole -Indent 2

		if (-not($PSDefaultParameterValues.ContainsKey("Start-BitsTransfer:ProxyAuthentication"))){
			Write-Log -Message "Prompting for credentials" -NoConsole -Indent 2
			$ProxyCredentials = Get-Credential -Message "Enter Proxy authentication credentials for $ProxyServer" -UserName "${env:USERDOMAIN}\${env:UserName}"
    	if ($ProxyCredentials){
    		Write-Log -Message "Credentials entered" -NoConsole -Indent 2
    		Write-Log -Message "Adding default values for Start-BitsTransfer" -NoConsole -Indent 2
    		Write-Log -Message "Adding default value for ProxyAuthentication - `"Basic`"" -NoConsole -Indent 2
    		$PSDefaultParameterValues.Add("Start-BitsTransfer:ProxyAuthentication","Basic")
    		$ProxyUserName = $ProxyCredentials.username
    		Write-Log -Message "Adding default value for ProxyCredential - `"$ProxyUserName`"" -NoConsole -Indent 2
    		$PSDefaultParameterValues.Add("Start-BitsTransfer:ProxyCredential",$ProxyCredentials)
    		# Write-Log -Message "Adding default value for ProxyList - `"$ProxyServer`"" -NoConsole -Indent 1
    		# $PSDefaultParameterValues.Add("Start-BitsTransfer:ProxyList",$ProxyServer)
    		# Write-Log -Message "Adding default value for ProxyUsage - `"AutoDetect`"" -NoConsole -Indent 1
    		# $PSDefaultParameterValues.Add("Start-BitsTransfer:ProxyUsage","AutoDetect")
			} else {
				Write-Log -Level error -message "Credentials NOT entered. Following commands will likely fail" -NoConsole -Indent 2
			}
    }
	} else {
		Write-Log -Message "Proxy is not enabled" -NoConsole -Indent 2
		# Clear the error queue in case the proxy registry key didn't exist at all and threw an error.
		$error.clear()
	}
} # end function Test-IsProxyEnabled

function Write-Log {
	<#
	.SYNOPSIS
		Extensive function to write data to either the console screen, a log file, and/or a Windows event log.

	.DESCRIPTION
		Extensive function to write data to either the console screen, a log file, and/or a Windows event log. Data can be written as info, warning, error, and includes indentation, time stamps, etc.

	.NOTES
	  Version							: 2.7
		Wish list						: Better error trapping
	  Rights Required			: Local administrator on server
	  Sched Task Required	: No
	  Lync Server Version	: N/A
	  Author/Copyright		:
	  Email/Blog/Twitter	: pat@innervation.com 	http://www.ehloworld.com @patrichard
	  Dedicated Post			:
	  Disclaimer   				: You running this script means you won't blame me if this breaks your stuff. This script is
	  											provided AS IS without warranty of any kind. I disclaim all implied warranties including,
	  											without limitation, any implied warranties of merchantability or of fitness for a particular
	  											purpose. The entire risk arising out of the use or performance of the sample scripts and
	  											documentation remains with you. In no event shall I be liable for any damages whatsoever
	  											(including, without limitation, damages for loss of business profits, business interruption,
	  											loss of business information, or other pecuniary loss) arising out of the use of or inability
	  											to use the script or documentation.
	  Acknowledgements 		: Test for log names and sources
													http://powershell.com/cs/blogs/tips/archive/2013/06/10/testing-event-log-names-and-sources.aspx
	  Assumptions					: ExecutionPolicy of AllSigned (recommended), RemoteSigned or Unrestricted (not recommended)
	  Limitations					:
	  Known issues				:

	.LINK


	.EXAMPLE
	  .\

	  Description
		-----------


	.INPUTS
		None. You cannot pipe objects to this script.
	#>
	[CmdletBinding(SupportsShouldProcess = $True, SupportsPaging = $True)]
	param(
		# The type of message to be logged. Alias is 'type'.
		[Parameter(Position = 0, ValueFromPipeline = $true, ValueFromPipelineByPropertyName = $true)]
		[ValidateSet("Error", "Warn", "Info")]
		[ValidateNotNullOrEmpty()]
		[string] $level = "Info",

		# The message to be logged.
		[Parameter(Position = 1, ValueFromPipeline = $true, ValueFromPipelineByPropertyName = $true, Mandatory = $true, HelpMessage = "No message specified.")]
		[ValidateNotNullOrEmpty()]
		[string] $Message,

		# Specifies that $message should not the sent to the log file.
		[Parameter(Position = 2, ValueFromPipeline = $false, ValueFromPipelineByPropertyName = $true)]
		[switch] $NoLog,

		# Specifies to not display the message to the console.
		[Parameter(Position = 3, ValueFromPipeline = $false, ValueFromPipelineByPropertyName = $true)]
		[switch] $NoConsole,

		# The number of spaces to indent the message in the log file.
		[Parameter(Position = 4, ValueFromPipeline = $false, ValueFromPipelineByPropertyName = $true)]
		[ValidateRange(1,30)]
		[ValidateNotNullOrEmpty()]
		[Int16] $Indent = 0,

		# Specifies what color the text should be be displayed on the console. Ignored when switch 'NoConsoleOut' is specified.
		[Parameter(Position = 5, ValueFromPipeline = $false, ValueFromPipelineByPropertyName = $true)]
		[ValidateSet("Black", "DarkMagenta", "DarkRed", "DarkBlue", "DarkGreen", "DarkCyan", "DarkYellow", "Red", "Blue", "Green", "Cyan", "Magenta", "Yellow", "DarkGray", "Gray", "White")]
		[ValidateNotNullOrEmpty()]
		[String] $ConsoleForeground = 'White',

		# Existing log file is deleted when this is specified. Alias is 'Overwrite'.
		[Parameter(Position = 6, ValueFromPipeline = $false, ValueFromPipelineByPropertyName = $true)]
		[Switch] $Clobber,

		# The name of the system event log, e.g. 'Application'. Note that writing to the system event log requires elevated permissions.
		[Parameter(Position = 7, ValueFromPipeline = $false, ValueFromPipelineByPropertyName = $true)]
		[ValidateSet("Application","System","Security","Lync Server","Microsoft Office Web Apps")]
		[ValidateNotNullOrEmpty()]
		[String] $EventLogName = "Application",

		# The name to appear as the source attribute for the system event log entry. This is ignored unless 'EventLogName' is specified.
		[Parameter(Position = 8, ValueFromPipeline = $false, ValueFromPipelineByPropertyName = $true)]
		[ValidateNotNullOrEmpty()]
		[String] $EventSource = $($MyInvocation.ScriptName).Name,

		# The ID to appear as the event ID attribute for the system event log entry. This is ignored unless 'EventLogName' is specified.
		[Parameter(Position = 9, ValueFromPipeline = $false, ValueFromPipelineByPropertyName = $true)]
		[ValidateRange(1,65535)]
		[ValidateNotNullOrEmpty()]
		[Int32] $EventID = 1,

		# The text encoding for the log file. Default is ASCII.
		[Parameter(Position = 10, ValueFromPipeline = $false, ValueFromPipelineByPropertyName = $true)]
		[ValidateSet("Unicode","Byte","BigEndianUnicode","UTF8","UTF7","UTF32","ASCII","Default","OEM")]
		[ValidateNotNullOrEmpty()]
		[String] $LogEncoding = "ASCII"
	)
	try {
		[string]$LogFolder = Split-Path $LogPath -Parent
		if (-not(Test-Path $LogFolder)){New-Item $LogFolder -type Directory | Out-Null}
		$msg = "{0} : {1} : {2}{3}" -f (Get-Date -Format "yyyy-MM-dd HH:mm:ss"), $Level.ToUpper(), ("  " * $Indent), $Message
		if (-not($NoConsole)){
			switch ($level) {
				"Error" {$Host.UI.WriteErrorLine("$Message")}
				"Warn" {Write-Warning $Message}
				"Info" {Write-Host $Message -ForegroundColor $ConsoleForeground}
			}
		}
		if (-not($NoLog)){
			if ($Clobber) {
				$msg | Out-File -FilePath $LogPath -Encoding $LogEncoding -Force
			} else {
				$msg | Out-File -FilePath $LogPath -Encoding $LogEncoding -Append
			}
		}

		# http://social.technet.microsoft.com/Forums/en-US/winserverpowershell/thread/e172f039-ce88-4c9f-b19a-0dd6dc568fa0/

		if ($EventLogName) {
			if (-not $EventSource) {
				[string] $EventSource = $([IO.FileInfo] $MyInvocation.ScriptName).Name
			}

			if(-not [Diagnostics.EventLog]::SourceExists($EventSource)) {
				[Diagnostics.EventLog]::CreateEventSource($EventSource, $EventLogName)
	    }

			switch ($Level) {
				"Error" {$EntryType = "Error"}
				"Warn"  {$EntryType = "Warning"}
				"Info"  {$EntryType = "Information"}
				Default  {$EntryType = "Information"}
			}
			Write-EventLog -LogName $EventLogName -Source $EventSource -EventId 1 -EntryType $EntryType -Message $Message
		}
		$msg = ""
	}
	catch {
		Throw "Failed to create log entry in: '$LogPath'. The error was: '$_'."
	}
} # end function Write-Log
#endregion Functions

if (-not(Test-Path "HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\{90150000-1151-0000-1000-0000000FF1CE}")){
	Write-Log -Level Error -Message "Office Web Apps Server (OWAS) is not installed on this server. Language packs cannot be installed until OWAS is installed."
	break
}
# check for auto updates
# deal with none to install


$AllLanguagePacks = Import-Csv -Path ".\OWASLanguagePacks.csv"
$NotInstalled = @()
foreach ($LanguagePack in $AllLanguagePacks){
	if (-not (Test-Path "HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\$($LanguagePack.GUID)")){
		$NotInstalled+= $LanguagePack
	}
}

Write-Log -Message "Presenting list of language packs that are not installed for selection" -NoConsole
$Selected = $NotInstalled | Select-Object Language | Out-GridView -OutputMode Multiple -Title "Pick the OWAS language packs you'd like to install"
Write-Log -Message "$($Selected.Count) language packs selected" -NoConsole -Indent 1
foreach ($selection in $Selected){
	$ThisLanguagePack = $AllLanguagePacks | Where-Object {$_.Language -eq $($Selection.Language)}
	Write-Log -Message "Installing $($ThisLanguagePack.Language) language pack"

	Write-Log -Message "Creating folder for $($ThisLanguagePack.Language) language pack" -NoConsole -Indent 1
  New-Item "$OWASLanguagePackFolder\$($ThisLanguagePack.Path)" -ItemType directory -ErrorAction SilentlyContinue | Out-Null

  if (-not(Test-Path "$OWASLanguagePackFolder\$($ThisLanguagePack.Path)\$(Split-Path -Path $ThisLanguagePack.URL -Leaf)")){
		Write-Log -Message "Downloading $($ThisLanguagePack.url)" -NoConsole -Indent 1
		# Start-BitsTransfer -Source $($ThisLanguagePack.URL) -Destination "$OWASLanguagePackFolder\$($ThisLanguagePack.Path)" -Description "Downloading $(Split-Path -Path $ThisLanguagePack.URL -Leaf)" -ErrorAction SilentlyContinue
		New-FileDownload -SourceFile $($ThisLanguagePack.URL) -DestFolder "$OWASLanguagePackFolder\$($ThisLanguagePack.Path)"
		$OWASFileVersion = $(((Get-Item $OWASLanguagePackFolder\$($ThisLanguagePack.Path)\$(Split-Path -Path $ThisLanguagePack.URL -Leaf)).VersionInfo).FileVersion)
  }else{
		Write-Log -Message "$OWASLanguagePackFolder\$($ThisLanguagePack.Path)\$(Split-Path -Path $ThisLanguagePack.URL -Leaf) already exists" -Indent 1
  }

  $OWASFilePath = "$OWASLanguagePackFolder\$($ThisLanguagePack.Path)\$($ThisLanguagePack.url | Split-Path -leaf)"
  if (Test-Path $OWASFilePath){
		if ($OWASFilePath.Substring($OWASFilePath.length -4) -eq ".img"){
			Write-Log -Message "This langauage pack is a disk image" -NoConsole -Indent 1
			Write-Log -Message "Mounting image $OWASFilePath" -NoConsole -Indent 1
			Get-DiskImage $OWASFilePath | Mount-DiskImage

			Write-Log -Message "Determining drive letter" -NoConsole -Indent 1
			$OWASLanguagePackSourceDrive = (Get-DiskImage $OWASFilePath | Get-Volume).DriveLetter + ":"

			Write-Log -Message "Starting silent installation of $($ThisLanguagePack.Language) language pack from $OWASLanguagePackSourceDrive" -NoConsole -Indent 1
			Start-Process -FilePath "$OWASLanguagePackSourceDrive\setup.exe" -ArgumentList "/config files\setupsilent\config.xml" -Wait

			Write-Log -Message "Dismounting $OWASLanguagePackSourceDrive" -NoConsole -Indent 1
			Get-DiskImage $OWASFilePath | Dismount-DiskImage
		}else{
			Write-Log -Message "This langauage pack is a compressed file" -NoConsole -Indent 1
			Write-Log -Message "Extracting language pack files" -NoConsole -Indent 1
			Start-Process -FilePath "$OWASLanguagePackFolder\$($ThisLanguagePack.Path)\$($ThisLanguagePack.url | Split-Path -leaf)" -ArgumentList "/extract:$OWASLanguagePackFolder\$($ThisLanguagePack.Path) /quiet" -wait

			Write-Log -Message "Starting silent installation of $($ThisLanguagePack.Language) language pack $OWASFileVersion from $OWASLanguagePackFolder\$($ThisLanguagePack.Path)\$($ThisLanguagePack.url | Split-Path -leaf)" -NoConsole -Indent 1
			Start-Process -FilePath "$OWASLanguagePackFolder\$($ThisLanguagePack.Path)\setup.exe" -ArgumentList "/config files\setupsilent\config.xml" -wait

      Write-Log -Message "Cleaning up extracted files" -NoConsole -Indent 1
      Get-ChildItem -Recurse -Path "$OWASLanguagePackFolder\$($ThisLanguagePack.Path)" | Where-Object {$_.Name -ne $($ThisLanguagePack.url | Split-Path -leaf)} | Remove-Item -Recurse -Force
		}
  }else{
    Write-Log -Level Error -Message "File not found"
  }
  if (Test-Path "HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\$($ThisLanguagePack.GUID)"){
  	Write-Log -Message "Completed installing $($ThisLanguagePack.Language) language pack version $((Get-ItemProperty -Path "HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\$($ThisLanguagePack.GUID)" -Name DisplayVersion).DisplayVersion)"
  }else{
  	Write-Log -Level Error -Message "An error has occurred"
  }
}
if ((Get-Service WACSM).Status -eq "Running"){
	Write-Log -Message "Restarting WACSM service"
	Write-Log -Message "Stopping WACSM service" -NoConsole -Indent 1
	Stop-Service WACSM
	Write-Log -Message "Starting WACSM service" -NoConsole -Indent 1
	Start-Service WACSM
}

# SIG # Begin signature block
# MIIOewYJKoZIhvcNAQcCoIIObDCCDmgCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUE21B1K1bi8stfcH354PSVZQG
# nSigggvAMIIFFTCCA/2gAwIBAgIQAkyGGvT5lv4ieQce2LzkEDANBgkqhkiG9w0B
# AQUFADBvMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYD
# VQQLExB3d3cuZGlnaWNlcnQuY29tMS4wLAYDVQQDEyVEaWdpQ2VydCBBc3N1cmVk
# IElEIENvZGUgU2lnbmluZyBDQS0xMB4XDTE0MDcxOTAwMDAwMFoXDTE1MTAwOTEy
# MDAwMFowazELMAkGA1UEBhMCVVMxCzAJBgNVBAgTAk1JMRkwFwYDVQQHExBTdGVy
# bGluZyBIZWlnaHRzMRkwFwYDVQQKExBJbm5lcnZhdGlvbiwgTExDMRkwFwYDVQQD
# ExBJbm5lcnZhdGlvbiwgTExDMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKC
# AQEApiLWbbmKubMqybnhPTgT5SI5FiWh13ALFlme3kydpK+CdYEIBiM3QoISo9kn
# oFv/aIUid1xLTA/bqlMSavBKxIctAoNB+FG/tRSvOlIRsOiu4rKro5eZfbCDIeuC
# eIbBG5OMylSySxb/LhaipyPKaoxiDSYh0S8XwMzU6+2t4e/V0I6opSadp8zh6FhV
# 2xyM5DbqdMLUj5pc15ipsPxWPuTiTnAMkAVPCU7qLXgwr9XJjR/lPuNhUe2tEZT7
# 7JYIqlq6G4eYPTa6fXfSzHpjAJPrpWtNAPPP0yjBnM8fEjDqjXG3HwazjVkteDL2
# Pb1yoJT0HSxCXSryVVfhoDVKOQIDAQABo4IBrzCCAaswHwYDVR0jBBgwFoAUe2jO
# KarAF75JeuHlP9an90WPNTIwHQYDVR0OBBYEFDHtO7hPm0uPBstUz9ZSAF+X2HWw
# MA4GA1UdDwEB/wQEAwIHgDATBgNVHSUEDDAKBggrBgEFBQcDAzBtBgNVHR8EZjBk
# MDCgLqAshipodHRwOi8vY3JsMy5kaWdpY2VydC5jb20vYXNzdXJlZC1jcy1nMS5j
# cmwwMKAuoCyGKmh0dHA6Ly9jcmw0LmRpZ2ljZXJ0LmNvbS9hc3N1cmVkLWNzLWcx
# LmNybDBCBgNVHSAEOzA5MDcGCWCGSAGG/WwDATAqMCgGCCsGAQUFBwIBFhxodHRw
# czovL3d3dy5kaWdpY2VydC5jb20vQ1BTMIGCBggrBgEFBQcBAQR2MHQwJAYIKwYB
# BQUHMAGGGGh0dHA6Ly9vY3NwLmRpZ2ljZXJ0LmNvbTBMBggrBgEFBQcwAoZAaHR0
# cDovL2NhY2VydHMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElEQ29kZVNp
# Z25pbmdDQS0xLmNydDAMBgNVHRMBAf8EAjAAMA0GCSqGSIb3DQEBBQUAA4IBAQBA
# 9scZLyK0WcZvB0iIDyBk9gQ88t7WI2koV+tVL7Unqc6+3XfQMFg4KSHqJZAobYtQ
# vlvn12wod5ZQzueEmLcB0eEj25IeQ9lp6yJZa2r4Li4rcVAaHdRsjh2COXLCbwGD
# W3X8uGUK4WwUT+VeH3/IMM8146Q98GOHtPJKZp/8JoTGrPLG86dL4S7P2+K+C8Qz
# +VSAVmCQDxl/YaCutxXzkP7tTt8mipNfqeuaGGLQBmhijHRry5c48Ftgl2vA1t9f
# r6adfEUyiJpfFHbR5ejuaIxATfRH79ofZ1O4DzMv5HftDSK9+lJoo9U88XfT8F/7
# sfZpBzUmMER912EezktLMIIGozCCBYugAwIBAgIQD6hJBhXXAKC+IXb9xextvTAN
# BgkqhkiG9w0BAQUFADBlMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQg
# SW5jMRkwFwYDVQQLExB3d3cuZGlnaWNlcnQuY29tMSQwIgYDVQQDExtEaWdpQ2Vy
# dCBBc3N1cmVkIElEIFJvb3QgQ0EwHhcNMTEwMjExMTIwMDAwWhcNMjYwMjEwMTIw
# MDAwWjBvMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYD
# VQQLExB3d3cuZGlnaWNlcnQuY29tMS4wLAYDVQQDEyVEaWdpQ2VydCBBc3N1cmVk
# IElEIENvZGUgU2lnbmluZyBDQS0xMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIB
# CgKCAQEAnHz5oI8KyolLU5o87BkifwzL90hE0D8ibppP+s7fxtMkkf+oUpPncvjx
# RoaUxasX9Hh/y3q+kCYcfFMv5YPnu2oFKMygFxFLGCDzt73y3Mu4hkBFH0/5OZjT
# O+tvaaRcAS6xZummuNwG3q6NYv5EJ4KpA8P+5iYLk0lx5ThtTv6AXGd3tdVvZmSU
# a7uISWjY0fR+IcHmxR7J4Ja4CZX5S56uzDG9alpCp8QFR31gK9mhXb37VpPvG/xy
# +d8+Mv3dKiwyRtpeY7zQuMtMEDX8UF+sQ0R8/oREULSMKj10DPR6i3JL4Fa1E7Zj
# 6T9OSSPnBhbwJasB+ChB5sfUZDtdqwIDAQABo4IDQzCCAz8wDgYDVR0PAQH/BAQD
# AgGGMBMGA1UdJQQMMAoGCCsGAQUFBwMDMIIBwwYDVR0gBIIBujCCAbYwggGyBghg
# hkgBhv1sAzCCAaQwOgYIKwYBBQUHAgEWLmh0dHA6Ly93d3cuZGlnaWNlcnQuY29t
# L3NzbC1jcHMtcmVwb3NpdG9yeS5odG0wggFkBggrBgEFBQcCAjCCAVYeggFSAEEA
# bgB5ACAAdQBzAGUAIABvAGYAIAB0AGgAaQBzACAAQwBlAHIAdABpAGYAaQBjAGEA
# dABlACAAYwBvAG4AcwB0AGkAdAB1AHQAZQBzACAAYQBjAGMAZQBwAHQAYQBuAGMA
# ZQAgAG8AZgAgAHQAaABlACAARABpAGcAaQBDAGUAcgB0ACAAQwBQAC8AQwBQAFMA
# IABhAG4AZAAgAHQAaABlACAAUgBlAGwAeQBpAG4AZwAgAFAAYQByAHQAeQAgAEEA
# ZwByAGUAZQBtAGUAbgB0ACAAdwBoAGkAYwBoACAAbABpAG0AaQB0ACAAbABpAGEA
# YgBpAGwAaQB0AHkAIABhAG4AZAAgAGEAcgBlACAAaQBuAGMAbwByAHAAbwByAGEA
# dABlAGQAIABoAGUAcgBlAGkAbgAgAGIAeQAgAHIAZQBmAGUAcgBlAG4AYwBlAC4w
# EgYDVR0TAQH/BAgwBgEB/wIBADB5BggrBgEFBQcBAQRtMGswJAYIKwYBBQUHMAGG
# GGh0dHA6Ly9vY3NwLmRpZ2ljZXJ0LmNvbTBDBggrBgEFBQcwAoY3aHR0cDovL2Nh
# Y2VydHMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElEUm9vdENBLmNydDCB
# gQYDVR0fBHoweDA6oDigNoY0aHR0cDovL2NybDMuZGlnaWNlcnQuY29tL0RpZ2lD
# ZXJ0QXNzdXJlZElEUm9vdENBLmNybDA6oDigNoY0aHR0cDovL2NybDQuZGlnaWNl
# cnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElEUm9vdENBLmNybDAdBgNVHQ4EFgQUe2jO
# KarAF75JeuHlP9an90WPNTIwHwYDVR0jBBgwFoAUReuir/SSy4IxLVGLp6chnfNt
# yA8wDQYJKoZIhvcNAQEFBQADggEBAHtyHWT/iMg6wbfp56nEh7vblJLXkFkz+iuH
# 3qhbgCU/E4+bgxt8Q8TmjN85PsMV7LDaOyEleyTBcl24R5GBE0b6nD9qUTjetCXL
# 8KvfxSgBVHkQRiTROA8moWGQTbq9KOY/8cSqm/baNVNPyfI902zcI+2qoE1nCfM6
# gD08+zZMkOd2pN3yOr9WNS+iTGXo4NTa0cfIkWotI083OxmUGNTVnBA81bEcGf+P
# yGubnviunJmWeNHNnFEVW0ImclqNCkojkkDoht4iwpM61Jtopt8pfwa5PA69n8SG
# nIJHQnEyhgmZcgl5S51xafVB/385d2TxhI2+ix6yfWijpZCxDP8xggIlMIICIQIB
# ATCBgzBvMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYD
# VQQLExB3d3cuZGlnaWNlcnQuY29tMS4wLAYDVQQDEyVEaWdpQ2VydCBBc3N1cmVk
# IElEIENvZGUgU2lnbmluZyBDQS0xAhACTIYa9PmW/iJ5Bx7YvOQQMAkGBSsOAwIa
# BQCgeDAYBgorBgEEAYI3AgEMMQowCKACgAChAoAAMBkGCSqGSIb3DQEJAzEMBgor
# BgEEAYI3AgEEMBwGCisGAQQBgjcCAQsxDjAMBgorBgEEAYI3AgEVMCMGCSqGSIb3
# DQEJBDEWBBTn50i5+Lw85wW/Gg3BJkDOb2x6bDANBgkqhkiG9w0BAQEFAASCAQCR
# +B2vNSodzOU/ocM/WfFZ+vAvkIjlmllz97TQp7P6DqbGb/dCDpxaHNTtOEQIRJpF
# CTxoWBj0dHry+g1yn9iE0qo5+AZssdavh2EjREkRiiHfFQ1BydnUCoIyXLA8dzwh
# Oqs6VaaZzcyTxtRI2OShmGwiKJ2PqzjAVANijMDJpM+NGU4xib0Jm81fTF/bDvsh
# GIFQoLYvvWDh1JIR4TanL8C3uqoBQwazz0+pHnFQgjV7ki0ms4fofFz9tMWehOA2
# 4ykfQKFjrp5Pi5Sf/vFoINE+HB2Hd+5npvovFKds0EEK9/s0L0shK10nbw9q2UTg
# MMrWeS1koPM93o+5RHT3
# SIG # End signature block
